/*
 * Copyright (C) 2020 Arnaud Ferraris <arnaud.ferraris@gmail.com>
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#define CALLAUDIO_DBUS_NAME "org.mobian_project.CallAudio"
#define CALLAUDIO_DBUS_PATH "/org/mobian_project/CallAudio"

#define CALLAUDIO_DBUS_TYPE G_BUS_TYPE_SESSION
